#!/usr/bin/perl -w

package RPM::Source::Convert;
use strict;
use warnings;

use DistroMap 0.05;
use RPM::Source::Transform;
our @ISA=q/RPM::Source::Transform/;

our (@PREHOOK_FIXES,@POSTHOOK_FIXES);

our $VERSION=0.29;

push @RPM::Source::Transform::DEFAULT_OPTIONS, @DistroMap::LONGOPT;

sub usage {
    my ($self)=@_;
    $self->SUPER::usage();
    print "\n";
    &DistroMap::longopt_usage();
}

sub get_distromap {
    my ($self)=@_;
    my $distromap=$self->{'__distromap'};
    return $distromap if $distromap;
    $self->{'__distromap'}=DistroMap->new();
    print STDERR "get_distromap: created distromap\n" if $RPM::Source::Transform::verbose>3;
    return $self->{'__distromap'};
}

sub callback_hooks {
    my ($self,$spec,$parent)=@_;
    print STDERR "entering prehook_fixes\n" if $RPM::Source::Transform::verbose>1;
    map {print STDERR $_->describe(1)} $spec->get_sections() if $RPM::Source::Transform::verbose>5;
    map {&$_($self,$spec,$parent)} @PREHOOK_FIXES;
    print STDERR "done prehook_fixes\n" if $RPM::Source::Transform::verbose>1;
    map {print STDERR $_->describe(1)} $spec->get_sections() if $RPM::Source::Transform::verbose>5;
}

sub callback_posthooks {
    my ($self,$spec,$parent)=@_;
    print STDERR "entering posthook_fixes\n" if $RPM::Source::Transform::verbose>1;
    map {print STDERR $_->describe(1)} $spec->get_sections() if $RPM::Source::Transform::verbose>5;
    map {&$_($self,$spec,$parent)} @POSTHOOK_FIXES;
    print STDERR "done posthook_fixes\n" if $RPM::Source::Transform::verbose>1;
    map {print STDERR $_->describe(1)} $spec->get_sections() if $RPM::Source::Transform::verbose>5;
}

1;

__END__


#### deprecated code, to be reused as generic unpacker for built-in lzma
use RPM::Source::Editor; # for run
sub _convert_source_from_zip {
    my ($spec, $source,$i)=@_;
    print STDERR "note: try to bzip2 zip source [$source]\n";
    my $sourcefile=$spec->{SOURCEDIR}."/".basename($source);
    if (! -e $sourcefile) {
	warn "Internal error: source $source not found!";
    } else {
	my $convertdir= $spec->{TMPDIR}.'/CONVERT'.$i;
	mkdir $convertdir;
	&RPM::Source::Editor::run ("pushd $convertdir; unzip -q $sourcefile; popd");
	my @files= map {s,^\Q$convertdir\E/,,; $_} glob ("$convertdir/*");
	my @hidden_files=grep {!/\.$/ && ! /\*$/} map {s,^\Q$convertdir\E/,,; $_} glob ("$convertdir/.*");
	my $arjsuff='.bz2';
	my $arjtarg='j';
	if (1) {
	    $arjtarg='';
	    $arjsuff='';
	}
	$sourcefile=~s/\.(zip|jar)$/.tar$arjsuff/;
	&RPM::Source::Editor::run ("tar c${arjtarg}f $sourcefile -C $convertdir @files @hidden_files");
	my $rawsource=$spec->raw_sourcelist_ref->[$i];
	my $newsource=$rawsource;
	$newsource=~s/\.(zip|jar)$/.tar$arjsuff/;
	my $main_sec=$spec->get_section('package','');
	$main_sec->subst(qr"$rawsource",$newsource);
    }
}
