#!/usr/bin/perl -w

package RPM::Source::Convert::ALTLinuxBackport;
use strict;
use warnings;
use Carp;
use Getopt::Long;
use RPM::Source::Transform 0.001;

#use RPM::Source::Convert;
use RPM::Source::Convert::DistroMap::Groups;
use RPM::Source::Convert::DistroMap::Packages;

our @ISA=q/RPM::Source::Convert/;

$RPM::Source::Transform::nextrel='altbackport';
$RPM::Source::Transform::NEXTREL_METHOD{'altbackport'}=\&nextrel_altbackport;
#$RPM::Source::Transform::INITREL_METHOD{'altbackport'}=\&nextrel_altbackport_init;
$RPM::Source::Transform::defaultchangelog='- backport';
$RPM::Source::Transform::force_transform_mode=1;

$DistroMap::default_originrepo='altlinux';
$DistroMap::default_originbranch='sisyphus';
$DistroMap::default_destrepo='altlinux';
$DistroMap::default_destbranch='p6';

our $backport_suffix='M60P';

push @RPM::Source::Transform::DEFAULT_OPTIONS,
    "A=s"   => \&__set_options,
    "M=s"   => \&__set_options;

sub __set_options {
    my ($bpclass,$suf)=@_;
    $backport_suffix=join('',$bpclass,$suf);
    my $destbranch=&_suffix2destbranch($backport_suffix);
    $DistroMap::default_destbranch=$destbranch;
    print STDERR "backport: destbranch=$destbranch suffix=$backport_suffix\n" if $RPM::Source::Transform::verbose>1;
}

sub usage {
    my ($self)=@_;
    $self->SUPER::usage();
    print "\n";
    print "ALT Linux backport specific options:\n";
    if ($Getopt::Long::bundling) {
	print "	-Mxx -- branch to backport. For example, -M60P.\n";
	print "	-Axx -- branch to autoport. For example, -A60T.\n";
    } else {
	print "	-M xx -- branch to backport. For example, -M 60P.\n";
	print "	-A xx -- branch to autoport. For example, -A 60T.\n";
    }
}

sub process_options {
    my ($self)=@_;
    $self->SUPER::process_options();
    $RPM::Source::Transform::defaultchangelog='- '.$self->__get_backport_suffix().' backport';
}

sub __get_backport_suffix() {
    my ($self)=@_;
    return $backport_suffix;
}

sub nextrel_altbackport {
    my ($rel,$spec,$parent)=@_;
    # $rel!~/\%{?release}?/; otherwise it is recurrent release invocation
    $spec->macros()->macro_subst_inplace(\$rel) if $rel=~/\%/ and $rel!~/\%{?release}?/;
    # $rel=~s/\%branch_release\s*//;
    # need after macro_subst_inplace
    $rel=~s/^\s*//;
    return &_nextrel_altbackport_raw($rel, $backport_suffix);
}

# sub nextrel_altbackport_init {
#     #my ($rel,$spec,$parent)=@_;
#     my ($spec,$parent)=@_;
#     my $rel=$spec->get_section('package','')->get_tag('Release');
#     return &_nextrel_altbackport_raw($rel, $backport_suffix);
# }

sub _nextrel_altbackport_raw {
    my ($rel, $backportsuffix)=@_;
    die "bad Release: [$rel] (does not start with alt - not an ALT Linux release !)" unless $rel=~/^(?:alt|\%)/;
    die "bad Release: [$rel] (does not have a digit - not an ALT Linux release !)" unless $rel=~/^(?:alt.*\d|\%).*$/;
    my $newrel = 'alt0.'.$backportsuffix.'.1';
    my ($relmajor,$relminor);
    if ($rel=~/^alt(.+\.|)(\d+)$/) {
	$relmajor=$1;
	$relminor=$2;
    } elsif($rel=~/^alt(.+)(\d+)$/) {
	$relmajor=$1;
	$relminor=$2;
    }
    if ($relminor) {
	$newrel = 'alt'.$relmajor.($relminor-1).'.'.$backportsuffix.'.1';
    }
    return $newrel;
}

sub _suffix2destbranch {
    my ($suffix)=@_;
    my $destbranch='p6';
    unless ($suffix=~/^([ABM])(.+)$/) {
	print STDERR "WARNING: unsupported suffix $suffix\n";
	return $destbranch;
    }
    my ($bpclass,$suf)=($1,$2);
    if ($suf eq '60' and $bpclass eq 'A') {
	$destbranch='t6';
    } elsif ($suf =~ '^([56789])0([PT])$') {
	$destbranch="\L$2\E${1}";
    } elsif ($suf =~ '^([5432])([012])$') {
	$destbranch="$1.$2";
    } else {
	print STDERR "WARNING: unsupported suffix $suffix\n";
    }
    return $destbranch;
}

sub _destbranch2suffix {
    my ($branch)=@_;
    my $suffix='M60P';
    if ($branch =~ '^([pt])([56789])$') {
	$suffix="M${2}0\U$1\E";
    } elsif ($branch =~ '^([5432])\.([012])$') {
	$suffix="M$1$2";
    } else {
	print STDERR "WARNING: unsupported branch $branch\n";
    }
    return $suffix;
}

1;
