#!/usr/bin/perl -w

package RPM::Source::Convert::DistroMap::Groups;
use strict;
use warnings;
use Carp;

use RPM::Source::Convert;

our %ALLOWED_GROUP;
# generic conversions
our %EXTRA_GROUP_POSTMAP = ();
# specific conversions
our %EXTRA_GROUP_PREMAP = ();

push @RPM::Source::Convert::PREHOOK_FIXES, \&distromap_group_fix;
unshift @RPM::Source::Convert::POSTHOOK_FIXES, \&distromap_group_posthook_fix;

if (-e '/usr/lib/rpm/GROUPS') {
    open my $rpmgroup, '<' , '/usr/lib/rpm/GROUPS' || die "can't open /usr/lib/rpm/GROUPS: $!";
    while (<$rpmgroup>) {
	chomp;
	s/\s*$//;
	$ALLOWED_GROUP{$_}=1 if $_;
    }
    close $rpmgroup;
}

sub distromap_group_posthook_fix {
    my ($self,$spec,$parent)=@_;
    foreach my $sec ($spec->get_sections()) {
	&distromap_group_postmap($self,$sec,$spec,$parent) if $sec->get_type eq 'package';
    }
}

sub distromap_group_fix {
    my ($self,$spec,$parent)=@_;
    foreach my $sec ($spec->get_sections()) {
	&distromap_group_premap($self,$spec,$sec) if $sec->get_type eq 'package';
    }
}

sub distromap_group_premap {
    my ($self,$spec,$section)=@_;
    my @out;
    my $distromap=$self->get_distromap;
    foreach (@{$section->get_bodyref()}) {
	if (/^Group:(\s*)(\S.*\S)\s*$/) {
	    my $gspace=$1;
	    my $group=$2;
	    my $group_premap=$distromap->unimap_exact_group_origin2dest($group);
	    $group_premap=$EXTRA_GROUP_PREMAP{$group} unless $group_premap;
	    if ($group_premap) {
		push @out, "Group:$gspace$group_premap\n";
	    } else {
		push @out, $_;
	    }
	} else {
	    push @out, $_;
	}
    }
    $section->set_body(\@out);
}

sub distromap_group_postmap {
    my ($self,$section,$spec,$parent)=@_;
    my @out;
    my $distromap=$self->get_distromap;
    foreach (@{$section->get_bodyref()}) {
	if (/^Group:(\s*)(\S.*\S)\s*$/) {
	    my $gspace=$1;
	    my $group=$2;
	    my $newgroup;
	    unless ($ALLOWED_GROUP{$group}) {
		if ($parent) {
		    my $parentsec=$parent->get_section('package',$section->get_raw_package());
		    $newgroup||=$parentsec->get_tag('Group') if $parentsec;
		}
		unless ($newgroup) {
		    my $group_postmap=$distromap->unimap_approximate_group_origin2dest($group);
		    $group_postmap=$EXTRA_GROUP_POSTMAP{$group} unless $group_postmap;
		    print STDERR "warning: substituting Group: $group with unaccurate $group_postmap\n" if $group_postmap;
		    $newgroup||=$group_postmap;
		}
		unless ($newgroup) {
		    my $sec_pkg_expanded = $section->get_package_name();
		    if ($sec_pkg_expanded =~/^python-module/) {
			&__try_group(\$newgroup,'Development/Python');
		    } elsif ($sec_pkg_expanded =~/^fonts-[ot]tf-/) {
			&__try_group(\$newgroup,'System/Fonts/True type');
		    } elsif ($sec_pkg_expanded =~/^fonts-type1-/) {
			&__try_group(\$newgroup,'System/Fonts/Type1');
		    } elsif ($sec_pkg_expanded =~/^fonts-bitmap-/) {
			&__try_group(\$newgroup,'System/Fonts/X11 bitmap');
		    }
		}
		print STDERR "WARNING: forbidden Group: $group in ".$section->get_raw_package().". Please, replace.\n" unless $newgroup;
	    }

	    if ($newgroup) {
		push @out, "Group:$gspace$newgroup\n";
	    } else {
		push @out, $_;
	    }
	} else {
	    push @out, $_;
	}
    }    
    $section->set_body(\@out);
}

sub __try_group {
    my ($ptr, $name)=@_;
    $$ptr=$name if $ALLOWED_GROUP{$name};
}

1;
