#!/usr/bin/perl -w

package RPM::Source::Convert::DistroMap::Packages;
use strict;
use warnings;
use Carp;

use RPM::Source::Convert;

push @RPM::Source::Convert::PREHOOK_FIXES, \&distromap_fix;

our %PROV_SKIP = map {$_ => 1} qw!!;
our %PROV_SUBST = ();

# todo: all is fedora-specific; move
our %DEP_DROP_VER = map {$_ => 1} qw!
libjpeg-devel
!;

our %DEP_SKIP = map {$_ => 1} qw!
%{__make}
/bin/rm /bin/ln
!;

our %DEP_SUBST = (
'%{_sbindir}/update-alternatives' => 'alternatives',
'/usr/sbin/update-alternatives' => 'alternatives',
'%{_sbindir}/alternatives' => 'alternatives',
'/usr/sbin/alternatives' => 'alternatives',
#'php-devel' => 'php5-devel',
#'php-embedded-devel' => 'php5-devel',
'tclx-devel' => 'tclx',
' ' => ' ',
);

sub distromap_fix {
    my ($self,$spec,$parent)=@_;
    foreach my $sec ($spec->get_sections()) {
	&distromap_cleanup_headers($self,$spec,$sec) if $sec->get_type() eq 'package';
    }
}

sub distromap_cleanup_headers {
    my ($self,$spec,$section)=@_;
    my @out;
    my $distromap=$self->get_distromap;
    my $is_prev_line_multiline=0;
    foreach (@{$section->get_bodyref()}) {
#	if (/^\s*((?:BuildReq|BuildRequires|PreReq|BuildPreReq|BuildPrereqs|Requires|Conflicts)\s*(?:\(\w+\))?)\s*:(\s*)(.*\S)\s*$/i) {
#%{?!_without_fuse:BuildRequires: fuse-devel}
	if (/^\s*(\%{\?!?[^:]+:)?\s*((?:BuildReq|BuildRequires|PreReq|BuildPreReq|BuildPrereqs|Requires|Conflicts)\s*(?:\(\w+\))?)\s*:(\s*)(.*\S)\s*$/i) {
	    my $macro=$1;
	    my $Tag=$2;
	    my $space=$3;
	    my $reqlist=$4;
	    my $endmacro='';
	    if ($macro) {
		$reqlist=~s/\s*(\})\s*$//;
		$endmacro=$1;
	    } else {
		$macro='';
	    }
	    my $filtered = &alt_filter_pkg_req_list($self,$spec,$reqlist);
	    push @out, "$macro$Tag:$space$filtered$endmacro\n" if ($filtered);
	} elsif (/^\s*(Provides)\s*:(\s*)(.*\S)\s*$/i) {
	    my $Tag=$1;
	    my $space=$2;
	    my $filtered = &alt_filter_pkg_prov_list($self,$spec,$3);
	    push @out, "$Tag:$space$filtered\n" if ($filtered);
	} else {
	    push @out, $_;
	}
	$is_prev_line_multiline= /\\\n$/;
    }
    $section->set_body(\@out);
}

sub alt_filter_pkg_prov_list {
    my ($self, $spec, $line)=@_;
    # hack around rpm4/add_changelog error
    #error: Dependency tokens must not contain '%<=>' symbols: Requires: icu4j-eclipse > = 3.6.1
    $line =~ s/\>\s+=/>=/g;
    $line =~ s/\<\s+=/<=/g;
    $line =~ s/\%\{dist\}//g;
    my @in = split /[\s,]+/, $line;
    my @out;
    while (@in) {
	$_ = shift @in;
	if ($PROV_SKIP{$_}) {
	    &read_tag_version(\@in);
	    next;
	} elsif ($PROV_SUBST{$_}) {
	    push @out, $PROV_SUBST{$_};
	#### those two below should be handled with filter subroutines ####
	} elsif (/^perl\((.+)\)$/) {
	    my $perlreq=$1;
	    if ($perlreq!~/\.p(?:h|l|m)$/) {
		# looks like fedora perl() provides
		$perlreq=~s!::!/!g;
		$perlreq.='.pm';
		$_='perl('.$perlreq.')';
	    }
	    push @out, $_;
	} elsif (/^font\(:lang=(.+)\)$/) {
	    my $langreq=$1;
	    push @out, 'fontlang('.$langreq.')';
	} else {
	    push @out, $_;
	}
    }
    return join(" ",@out);
};

sub alt_filter_pkg_req_list {
    my ($self, $spec, $line)=@_;
    my $distromap=$self->get_distromap;
    # hack around rpm4/add_changelog error
    #error: Dependency tokens must not contain '%<=>' symbols: Requires: icu4j-eclipse > = 3.6.1
    $line =~ s/\>\s+=/>=/g;
    $line =~ s/\<\s+=/<=/g;
    $line =~ s/\%\{dist\}//g;
    my @in = split /[\s,]+/, $line;
    my @out;
    while (@in) {
	local $_ = shift @in;
	my $mapname=join(' ',$distromap->multimap_binary_origin2dest($spec->macros()->macro_subst($_)));
	if ($RPM::Source::Transform::verbose>1) {
	    print STDERR "distromap: $_ -> $mapname\n" if $mapname;
	    print STDERR "distromap: no map for $_\n" unless $mapname;
	}
	if ($DEP_SKIP{$_}) {
	    &read_tag_version(\@in);
	    next;
	} elsif ($DEP_SUBST{$_}) {
	    push @out, $DEP_SUBST{$_};
	    &read_tag_version(\@in) if $DEP_DROP_VER{$_};
	} elsif ($mapname and $mapname ne $_) {
	    push @out, $mapname;
	    &read_tag_version(\@in) if $DEP_DROP_VER{$_};
	} elsif ($DEP_DROP_VER{$_}) {
	    push @out, $_;
	    &read_tag_version(\@in);
	#### those two below should be handled with filter subroutines ####
	} elsif (/^perl\((.+)\)$/) {
	    my $perlreq=$1;
	    if ($perlreq!~/\.p(?:h|l|m)$/) {
		# looks like fedora perl() provides
		$perlreq=~s!::!/!g;
		$perlreq.='.pm';
		$_='perl('.$perlreq.')';
	    }
	    push @out, $_;
	} elsif (/^font\(:lang=(.+)\)$/) {
	    my $langreq=$1;
	    push @out, 'fontlang('.$langreq.')';
	} else {
	    push @out, $_;
	}
    }
    return join(" ",@out);
};

sub read_tag_version {
    my $ptr=shift;
    return if ($#{$ptr}<1);
    if ($ptr->[0] =~/^(?:<|>|=|!=|>=|<=)$/) {
	return (shift @$ptr, shift @$ptr);
    }
    return;
}

1;
