#!/usr/bin/perl -w

package RPM::Source::Convert::Fedora2ALT;
use strict;
use warnings;
#use Carp;

#use RPM::Source::Convert;
use RPM::Source::Convert::Generic2ALT;

# fake section to be processed in &convert_font_pkg
# a method to protect multisection macro from killing as the part of %clean section
$RPM::Source::SpecSection::SECTION_MACROS{'_font_pkg'}=1;

# so, the order is not so simple
unshift @RPM::Source::Convert::PREHOOK_FIXES, \&convert_font_pkg;
push @RPM::Source::Convert::PREHOOK_FIXES, \&fix;

unshift @RPM::Source::Convert::POSTHOOK_FIXES, \&posthook_fix;

my $FEDORA_VERSION=15;

sub fix {
    my ($self,$spec,$parent)=@_;

    my @sections=$spec->get_sections();
    my $mainsec=$sections[0];

    $mainsec->subst(qr'\%{x86}','%{ix86}',qr'^ExclusiveArch:');

    my ($flag_macro_fedora, $flag_macro_isa_bits, $flag_macro_texmf);
    my ($flag_macro_bcond_with, $flag_need_fedora_compat);
    foreach my $sec (@sections) {
	my $t = $sec->get_type();
	if ($t eq 'package') {
	    #
	} elsif ($t eq 'build') {
	    $sec->subst(qr'^.\s+/etc/profile.d/qt.sh','. /etc/profile.d/qt3dir.sh');

	    if ($sec->match_body('(?<!%)\%{?cmake}?(?=\s|$)')) {
		$flag_need_fedora_compat=1;
		$sec->subst(qr'(?<!%)\%{?cmake}?(?=\s|$)','%{fedora_cmake}');
	    }
	    $sec->subst(qr'\%{?_qt4_qmake}?','qmake-qt4');
	    if ($sec->match_body('^\s*qmake(?:\s|$)')) {
		my $qtver=3;
		$qtver=4 if $mainsec->match_body('^BuildReq.+qt4-devel');
		$sec->subst(qr'^\s*qmake(?=(?:\s|$))','qmake-qt'.$qtver);
	    }
	} elsif ($t eq 'install') {
	    # TnL-data, for example
	    if ($sec->match_body(qr'ln.+/fonts/dejavu/')) {
		$sec->subst_if('/fonts/dejavu/','/fonts/ttf/dejavu/',qr'\.ttf');
	    }
	} elsif ($t eq 'files') {
	    #
	}
	# general hacks here
	# is it worth wrapping in one?
	$sec->subst(qr'%{_localstatedir}/', '%{_var}/');
	# altlinux vs redhat incompatibility (found in hsqldb)
	#$sec->subst(qr'%{_localstatedir}/lib', '%{_var}/lib');
	#$sec->subst(qr'%{_localstatedir}/log', '%{_var}/log');# jetty 5
	#$sec->subst(qr'%{_localstatedir}/run', '%{_var}/run');# jetty 5
	#$sec->subst(qr'%{_localstatedir}/cache', '%{_var}/cache');# jetty 5
	# due to feature of alt findreq
	$sec->subst(qr'%{_sysconfdir}/init.d','%_initdir');

	$sec->subst(qr'%{_emacs_sitelispdir}','%{_emacslispdir}');
	$sec->subst(qr'%_emacs_sitelispdir','%_emacslispdir');
	$sec->subst(qr'\%{_tmppath}','%{_var}/tmp');

	# TODO: python
	#$sec->subst(qr'%_python_sitelib','%_python_sitelibdir');

	if ($sec->match_body(qr'(?<!%)\%{?_texmf_main}?')) {
	    $sec->subst(qr'%_texmf_main','%_texmfmain');
	    $sec->subst(qr'%{_texmf_main}','%{_texmfmain}');
	    $flag_macro_texmf=1;
	}
	$flag_macro_fedora=1 if not $flag_macro_fedora and $sec->match_body(qr'(?<!%)\%{\??fedora}');
	$flag_macro_isa_bits=1 if not $flag_macro_isa_bits and $sec->match_body(qr'(?<!%)\%{__isa_bits}');
	$flag_macro_bcond_with=1 if not $flag_macro_bcond_with and $sec->match_body(qr'(?<!%)\%bcond_with');
	$flag_need_fedora_compat=1 if not $flag_need_fedora_compat and $sec->match_body(qr'(?<!%)\%{?_unitdir}?');
    }
    $mainsec->unshift_body('BuildRequires(pre): rpm-macros-fedora-compat'."\n") if $flag_need_fedora_compat;
    $mainsec->unshift_body('BuildRequires(pre): rpm-build-texmf'."\n") if $flag_macro_texmf;
    $mainsec->unshift_body('%define fedora '.$FEDORA_VERSION."\n") if $flag_macro_fedora;
    $mainsec->unshift_body('# fedora __isa_bits tmp hack
%ifarch x86_64
%define __isa_bits 64
%else
%define __isa_bits 32
%endif
') if $flag_macro_isa_bits;
    $mainsec->unshift_body('# fedora bcond_with macro
%define bcond_with() %{expand:%%{?_with_%{1}:%%global with_%{1} 1}}
%define bcond_without() %{expand:%%{!?_without_%{1}:%%global with_%{1} 1}}
') if $flag_macro_bcond_with;
}


sub posthook_fix {
    my ($self,$spec,$parent)=@_;
    my @sections=$spec->get_sections();
    my $mainsec=$sections[0];
    foreach my $sec (@sections) {
	my $t = $sec->get_type();
	if ($t eq 'package') {
	} elsif ($t eq 'build') {
	} elsif ($t eq 'install') {
	    if ($self->{'__FEDORA::FONT::PKG__'}) {
		if (not $spec->{'CONVERT::FEDORA::NO_MOVE_FONTS'}) {
		    $sec->push_body(q!# generic fedora font import transformations
# move fonts to corresponding subdirs if any
for fontpatt in OTF TTF TTC otf ttf ttc pcf pcf.gz afm pfa pfb; do
    case "$fontpatt" in 
	pcf*) type=bitmap;;
	tt*|TT*) type=ttf;;
	otf|OTF) type=otf;;
	afm*|pf*) type=type1;;
    esac
    find $RPM_BUILD_ROOT/usr/share/fonts -type f -name '*.'$fontpatt | while read i; do
	j=`echo "$i" | sed -e s,/usr/share/fonts/,/usr/share/fonts/$type/,`;
	install -Dm644 "$i" "$j";
	rm -f "$i";
	olddir=`dirname "$i"`;
	mv -f "$olddir"/{encodings.dir,fonts.{dir,scale,alias}} `dirname "$j"`/ 2>/dev/null ||:
	rmdir -p "$olddir" 2>/dev/null ||:
    done
done
!);
		}
		if (not $spec->{'CONVERT::FEDORA::NO_RELINK_FONTCATALOG'}) {
			$sec->push_body(q!# kill invalid catalogue links
if [ -d $RPM_BUILD_ROOT/etc/X11/fontpath.d ]; then
    find -L $RPM_BUILD_ROOT/etc/X11/fontpath.d -type l -print -delete ||:
    # relink catalogue
    find $RPM_BUILD_ROOT/usr/share/fonts -name fonts.dir | while read i; do
	pri=10;
	j=`echo $i | sed -e s,$RPM_BUILD_ROOT/usr/share/fonts/,,`; type=${j%%%%/*}; 
	pre_stem=${j##$type/}; stem=`dirname $pre_stem|sed -e s,/,-,g`;
	case "$type" in 
	    bitmap) pri=10;;
	    ttf|ttf) pri=50;;
	    type1) pri=40;;
	esac
	ln -s /usr/share/fonts/$j $RPM_BUILD_ROOT/etc/X11/fontpath.d/"$stem:pri=$pri"
    done ||:
fi
!);
		}
	    }
	} elsif ($t eq 'files') {
	    $sec->subst(qr'%{_fontdir}','%{_fontbasedir}/*/%{_fontstem}')
		if ($self->{'__FEDORA::FONT::PKG__'});
	    $sec->apply_body(sub {
		local $_=$_[0];
		if (m!^(.*)\%{catalogue}/(\S+)\s*$!) {
		    my $pathbase=$1;
		    my $stem=$2;
		    $stem=~s/-fonts$//;
		    $_=$pathbase.'%{catalogue}/'.$stem."*\n" unless /\*\s*$/;
		}
		return $_;
	    });
	}
    }
}


sub convert_font_pkg {
    my ($self,$spec,$parent)=@_;
    my $distromap=$self->get_distromap;
    # first search for %_font_pkg
    my $flag_font_pkg_processed=0;
    my @sections=$spec->get_sections();
    my $mainsec=$sections[0];
    return unless $mainsec->match_body(qr'Build(?:Pre)?Req.*fontpackages-devel');    my $font_type=&__get_font_type_from_name('',$distromap->unimap_source_origin2dest($spec->macros()->macro_subst('%name')));
    foreach my $sec (@sections) {
	$sec->apply_body(sub {
	    local $_=shift;
	    unless (/^\s*\%_font_pkg/) {
		return $_;
	    } else {
		#print STDERR "Yahoo! \%_font_pkg detected: $_";
		$flag_font_pkg_processed=1;
		chomp;
		s/^\s*\%_font_pkg\s*//;
		my @out='%files'."\n";
		my @filepattern;
		while (s/^(\S+)\s*//) {
		    if ($1 eq '-f') {
			s/^(\S+)\s*//;
			push @out, '%{_fontconfig_templatedir}/'."$1\n";
			push @out, '%config(noreplace) %{_fontconfig_confdir}/'."$1\n";
		    } elsif ($1 eq '-n') {
			s/^(\S+)\s*//;
			my $fontstem=$1;
			my $fcname='%{?fontname:%fontname}%{!?fontname:%name}-'.$fontstem.'-fonts';
			# mplus-fonts
			$fcname=$fontstem.'-fonts' if ($fontstem =~/fontname/);
			my @mapname=$distromap->multimap_binary_origin2dest($spec->macros()->macro_subst($fcname));
			$font_type=&__get_font_type_from_name($font_type,@mapname);
			$mapname[0]||=$fcname;
			$out[0] ='%files -n '.$mapname[0]."\n";
		    } else {
			push @filepattern, $1;
		    }
		}
		s/\s*$//;
		push @filepattern, $_ if $_;
		# does not work for multi packages
		#push @out, '%{_fontbasedir}/*'."\n";
		push @out, map { '%{_fontbasedir}/*/%{_fontstem}/'.$_."\n"} @filepattern;
		return @out;
	    }
			 }, '%_font_pkg converter');
    }
    # not really needed; fontdir (not _fontdir) is used in few packages? baekamuk-bdf-fonts
    #$sections->push_body('%define fontdir %{_datadir}/fonts/'.$font_type.'/%{_fontstem}'."\n") if $font_type;
    $spec->_reset_speclist() if $flag_font_pkg_processed;
    $self->{'__FEDORA::FONT::PKG__'}=1; #if $flag_font_pkg_processed;
}

sub __get_font_type_from_name {
    my ($font_type,@mapname)=@_;
    if (@mapname and $mapname[0] =~ /^fonts-(ttf|otf|type1|bitmap)-/) {
	if ($font_type) {
	    $font_type='default' if $font_type ne $1;
	} else {
	    $font_type=$1;
	}
    }
    return $font_type;
}

1;
__END__
