#!/usr/bin/perl -w

package RPM::Source::Convert::MassALTLinuxBackport;

use strict;
use warnings;
use Carp;
use RPM qw/evrcmp/;

use RPM::Source::Convert::Mass;
use RPM::Source::Convert::ALTLinuxBackport;

our @ISA = qw/RPM::Source::Convert::Mass Exporter/;
our @EXPORT = qw/$DESTPATH $ORIGINPATH @ORIGINSKIP @DESTSKIP %ORIGIN2DESTMAP/;

our $backport_suffix;
our $destbranch='p6';

$RPM::Source::Convert::Mass::strategy = 'newer';
$RPM::Source::Convert::Mass::originimport='srpmbackport';
$RPM::Source::Convert::Mass::suppress_rename=1;

$DistroMap::default_originrepo='altlinux';
$DistroMap::default_originbranch='sisyphus';
$DistroMap::default_destrepo='altlinux';

sub new {
    my $class = shift;
    $destbranch=&RPM::Source::Convert::ALTLinuxBackport::_suffix2destbranch($backport_suffix) if $backport_suffix;
    $DistroMap::default_destbranch=$destbranch;
    return RPM::Source::Convert::Mass::new(
	$class,
	@_);
};

sub process_options_hook {
    my ($self)=@_;
    $backport_suffix=&RPM::Source::Convert::ALTLinuxBackport::_destbranch2suffix($DistroMap::default_destbranch) unless $backport_suffix;
}

sub origin_import_hook {
    my ($self,$argsptr)=@_;
    push @$argsptr, '-'.$RPM::Source::Convert::MassALTLinuxBackport::backport_suffix;
}

sub origin_cmp_dest {
    my ($self,$origin_rh,$dest_rh) =@_;
    return &__backport_origin_cmp_dest($origin_rh,$dest_rh);
}

sub __backport_origin_cmp_dest {
    my ($origin_rh,$dest_rh) =@_;
    return 1 unless $dest_rh;
    return -1 unless $origin_rh;
    my $originversion=$origin_rh->{VERSION};
    my $originrelease=$origin_rh->{RELEASE};
    my $destversion=$dest_rh->{VERSION};
    my $destrelease=$dest_rh->{RELEASE};
    my $autoportsrelease=&RPM::Source::Convert::ALTLinuxBackport::_nextrel_altbackport_raw($originrelease, $backport_suffix);
    my $cmp = $originversion cmp $destversion;
    if ($cmp!=0) {
	$cmp=evrcmp($originversion,$destversion);
	print STDERR "version: $originversion rpmcmp $destversion = $cmp\n" if $RPM::Source::Convert::Mass::verbose>1;
    }
    return $cmp if $cmp !=0;
    return 0 if $destrelease eq $autoportsrelease or $destrelease eq $originrelease;
    # Binary "cmp" returns -1, 0, or 1 depending on whether the left argument
    # is stringwise less than, equal to, or greater than the right argument.
    $cmp=evrcmp($originrelease,$destrelease);
    print STDERR "$originrelease rpmcmp $destrelease = $cmp\n" if $RPM::Source::Convert::Mass::verbose>1;
    return $cmp if $cmp <= 0;
    $cmp=evrcmp($autoportsrelease,$destrelease);
    print STDERR "$autoportsrelease rpmcmp $destrelease = $cmp\n" if $RPM::Source::Convert::Mass::verbose>1;
    return $cmp;
}

1;
