#!/usr/bin/perl -w

use strict;
use warnings;
use Carp;
use Getopt::Long;
use File::Basename;
use File::Path qw(make_path remove_tree);
use RPM::Header;
use RPM qw/evrcmp/;

my $PATH='/var/ftp/pub/Linux/fedora/linux/development/rawhide/source/SRPMS';

my $help;
our $verbose=0;
my $result = GetOptions (
    "help"		=> \$help,
    "verbose|v+"	=> \$verbose,
    );

my @all_names;
my %ignore;
my %selected;

sub usage {
	print "usage [ options ] [ list of origin rpms/names ]
options:
";
}

my $base=$ARGV[0];

my $opt_taskfile="import/fc/$base.glob";
my $opt_ignorefile="import/fc/$base.ignore";
$opt_ignorefile=undef if not -e $opt_ignorefile;


if ($opt_ignorefile) {
    open (my $fn, '<', $opt_ignorefile) or die "can't open file $opt_ignorefile\n";
    while (my $line=<$fn>) {
	my @list = &__line2array($line);
	$ignore{$list[0]}=1 if defined $list[0];
    }
    close($fn);
}
if ($opt_taskfile) {
    my @patterns;
    open (my $fn, '<', $opt_taskfile) or die "can't open file $opt_taskfile\n";
    while (my $line=<$fn>) {
	my @list = &__line2array($line);
	push @patterns, @list;
	push @all_names, &find_namelist(@list);
    }
    close($fn);
    map {$selected{$_}=1 if not $ignore{$_}} map {&__expand_glob($_,\@all_names)} @patterns;
}

map {print "$_\n"} sort keys (%selected);


# glob expansion on $opt_namefile
sub __expand_glob {
    my ($pattern, $arrayptr)=@_;
    return $pattern unless $pattern =~/[?*]/;
    my $regexp=&glob2pat($pattern);
    my @names = grep {/$regexp/} @$arrayptr;
    print STDERR "WARNING: pattern $pattern does not match.\n" unless @names;
    return @names;
}

sub __line2array {
    my $line=shift;
    chomp $line;
    $line=~s/^\s*//;
    $line=~s/\s*$//;
    return if $line=~/^\s*#/;
    return if $line eq '';
    return split (/\s+/, $line);
}

sub glob2pat {
    my $globstr = shift;
    my %patmap = (
	'*' => '.*',
	'?' => '.',
	'[' => '[',
	']' => ']',
	'{' => '(?:',
	'}' => ')',
	',' => '|',
	);
    $globstr =~ s{(.)} { $patmap{$1} || "\Q$1" }ge;
    return '^' . $globstr . '$';
}

sub find_namelist {
    my $namepat=shift;
    my $cur_href;
    my @names;
    return unless defined $namepat;
    foreach my $rpmfile (glob "$PATH/$namepat-*.src.rpm") {
	my $href = new RPM::Header $rpmfile;
	push @names, $href->{NAME};
    }
    return @names;
}

1;
